import os
import sys
sys.path.append(r"C:\Users\Cheny\Documents\Python\PyPi_DoNotDelete\clioedb")
from clioedb.mydbconn import Connect_DB
filename = os.path.basename(__file__)
log_name = filename.split(".")[0] + '.log'

db_credential = {  #
    "user": "root",
    "password": "Belajar1",
    "host": "127.0.0.1",
    "db": "sql_store",
}
try:
    db = Connect_DB(db_credential)
    sql = "SELECT * FROM sql_store.customers;"
    rc = db.fetchone(sql)
    print(rc)
finally:
    db.close



import os


def write_list_file(list_data,  output_filename, mode="w"):
    """ Write the given list to text file

    parameters:
    list_data (list): data as list
    output_filename (str): output file name, ie. output.txt 
    mode(str): file stream mode with default = "w"
    """
    try:
        ori_umask = os.umask(0)
        with open(output_filename, mode, os.O_RDWR | os.O_CREAT) as f:  # output file name
            if list_data:
                f.writelines(list_data)
    finally:
        os.umask(ori_umask)


def write_dbdata_file(data_as_list, output_filename, mode="w", header_as_list=[], delimiter="\t"):
    """ Export the data (list of tuples) to text file for records.

    parameters:
    data_as_list (list of tuples): row of data in tuples ie. [(row1),(row2) .. (rown)]
    output_filename (str): output file name, ie. output.txt 
    header_as_list (list): header as list if required, ie. ['column1', 'column2',  'column3'.. 'columnN']
    mode(str): file stream mode with default = "w"
    delimiter (str): default as tab delimiter
    """
    try:
        ori_umask = os.umask(0)
        with open(output_filename, mode, os.O_RDWR | os.O_CREAT) as fw:
            if len(header_as_list) > 0:  # if header
                fw.write(delimiter.join(str(item)
                                        for item in header_as_list)+"\n")
            for line in data_as_list:
                fw.write(delimiter.join(str(item) for item in line) + "\n")
    finally:
        os.umask(ori_umask)



import logging
import MySQLdb


def set_logger(log_name):
    """ setting up logger for recording database execution with format %(asctime)s: %(levelname)s: %(message)s
    parameters:
    log_name (str): logger file name ie. logfile.log

    returns: logging.Logger object
    """
    logger = logging.getLogger()
    logger.setLevel(logging.INFO)
    formatter = logging.Formatter('%(asctime)s: %(levelname)s: %(message)s')
    file_handler = logging.FileHandler(log_name)
    file_handler.setFormatter(formatter)
    logger.addHandler(file_handler)
    return log_name


class Connect_DB:
    """ Blueprint to create MySQLdb connection object with logger. 
    the benefit of using this class that it will create a log file for each execution.

    parameters:
    my_logger (logging.Logger) with default = None
    """

    def __init__(self, db_credential: dict, my_logger: logging.Logger = None):
        self.logger = my_logger
        self.conn = None
        self.cursor = None
        self.credential = db_credential
        self.__connect(db_credential)

    def __connect(self, db_credential: dict):  # private
        self.conn = MySQLdb.connect(**db_credential)
        self.conn.ssl_mode = "DISABLED"
        self.conn.ping(True)

        self.cursor = self.conn.cursor(MySQLdb.cursors.DictCursor)
        self.conn.autocommit(True)
        if self.logger:
            self.logger.info(
                f"Connection to {db_credential['database']} has been created.")

    def execute(self, sql: str, data=None):
        if self.logger:
            text = f"{sql} {data}" if data else f"{sql}"
            self.logger.info(text)
        return self.cursor.execute(sql, data)

    def fetchall(self, sql: str, data=None):
        self.execute(sql, data)
        return self.cursor.fetchall()

    def fetchone(self, sql: str, data=None):
        self.execute(sql, data)
        return self.cursor.fetchone()

    def update(self, sql: str, data=None):
        rows_count = self.execute(sql, data)
        self.conn.commit()
        if self.logger:
            self.logger.info(f"Updated {rows_count} rows.")
        return rows_count

    def insert(self, sql: str, data=None):
        self.execute(sql, data)
        last_row_id = self.cursor.lastrowid
        self.conn.commit()
        if self.logger:
            self.logger.info(f"Insert id = {last_row_id}")
        return last_row_id

    def close(self):
        if self.conn.open:
            self.conn.close()
            self.conn = None
        if self.logger:
            self.logger.info(
                f"Close connection to {self.credential['database']}.")



import MySQLdb
import os


def fetch_dbdata_list(sql, db_credential, header=True):
    """ Fetch data from database from the given sql and db_credential

    parameters:
    sql (str): mysql query
    db_credential (dict): database credential
    ie. 
    db_credential = {  #
        "user": "admin",
        "password": "password",
        "host": "127.0.0.1",
        "db": "mydb",
    }
    header (bool) with default as True

    returns list of data tuples then you can export the data to files using export_data_file
    i.e.:   return_data = export_db_list(sql, header=True)
            export_data_file(return_data, output_filename, mode="w", header_as_list=[], delimiter="\t")
    """
    try:
        header_list = []
        data_list = []  # list of list
        conn = MySQLdb.connect(**db_credential)
        cursor = conn.cursor(MySQLdb.cursors.DictCursor)
        cursor.execute(sql)
        record_set = cursor.fetchall()
        count = 0
        for record in record_set:
            count += 1
            if (header and count == 1):
                header_list.append(record.keys())
            data_list.append(record.values())
        return header_list + data_list  # combined header w/ data
    finally:
        conn.close()


# direct output to file (one solution)
def fetch_dbdata_file(sql, db_credential, output_filename, mode="w", delimiter="\t"):
    """ Fetch data from database from the given sql and db_credential and save it as file

    parameters:
    sql (str): mysql query
    db_credential (dict): database credential
    ie. 
    db_credential = {  #
        "user": "admin",
        "password": "password",
        "host": "127.0.0.1",
        "db": "mydb",
    }
    output_filename (str): output file name, ie. output.txt 
    mode(str): file stream mode with default = "w"
    delimiter (str): default as tab delimiter

    returns dbdata as text file
    """
    try:
        conn = MySQLdb.connect(**db_credential)
        cursor = conn.cursor(MySQLdb.cursors.DictCursor)
        cursor.execute(sql)
        record_set = cursor.fetchall()
        count = 0
        ori_umask = os.umask(0)
        with open(output_filename, mode, os.O_RDWR | os.O_CREAT) as fw:
            for record in record_set:
                count += 1
                if (count == 1):
                    fw.write(delimiter.join(str(item)
                                            for item in record.keys())+'\n')
                fw.write(delimiter.join(str(item)
                         for item in record.values())+'\n')
    finally:
        os.umask(ori_umask)
        conn.close()




import setuptools
from pathlib import Path

setuptools.setup(
    name="clioedb",
    version=1.6,
    description="A module to fetch data from MySQL db as list or save it as text file",
    keywords=["fetch mysql data"],
    long_description=Path("README.md").read_text(),
    author='Cheny Lioe',
    author_email='chenylioe@yahoo.com',
    packages=setuptools.find_packages(exclude=["tests", "data"])
)
